"ID"<@movies_image>,"@TITLE@"</@movies_image><@movies_title>,"@TITLE@"</@movies_title><@movies_o_title>,"@TITLE@"</@movies_o_title><@movies_number>,"@TITLE@"</@movies_number><@movies_year>,"@TITLE@"</@movies_year><@movies_director>,"@TITLE@"</@movies_director><@movies_rating>,"@TITLE@"</@movies_rating><@movies_runtime>,"@TITLE@"</@movies_runtime><@movies_country>,"@TITLE@"</@movies_country><@movies_genre>,"@TITLE@"</@movies_genre><@movies_site>,"@TITLE@"</@movies_site><@movies_o_site>,"@TITLE@"</@movies_o_site><@movies_trailer>,"@TITLE@"</@movies_trailer><@movies_media_num>,"@TITLE@"</@movies_media_num><@movies_seen>,"@TITLE@"</@movies_seen><@movies_loaned>,"@TITLE@"</@movies_loaned><@movies_classification>,"@TITLE@"</@movies_classification><@movies_studio>,"@TITLE@"</@movies_studio><@movies_cast>,"@TITLE@"</@movies_cast><@movies_plot>,"@TITLE@"</@movies_plot><@movies_notes>,"@TITLE@"</@movies_notes><@vcodecs_name>,"@TITLE@"</@vcodecs_name>
<!-- ITEMS --><@item>"@DATA@"</@item><@movies_image><@item>,</@item>"@DATA@"</@movies_image><@movies_title>,"@DATA@"</@movies_title><@movies_o_title>,"@DATA@"</@movies_o_title><@movies_number>,"@DATA@"</@movies_number><@movies_year>,"@DATA@"</@movies_year><@movies_director>,"@DATA@"</@movies_director><@movies_rating>,"@DATA@"</@movies_rating><@movies_runtime>,"@DATA@"</@movies_runtime><@movies_country>,"@DATA@"</@movies_country><@movies_genre>,"@DATA@"</@movies_genre><@movies_site>,"@DATA@"</@movies_site><@movies_o_site>,"@DATA@"</@movies_o_site><@movies_trailer>,"@DATA@"</@movies_trailer><@movies_media_num>,"@DATA@"</@movies_media_num><@movies_seen>,"@DATA@"</@movies_seen><@movies_loaned>,"@DATA@"</@movies_loaned><@movies_classification>,"@DATA@"</@movies_classification><@movies_studio>,"@DATA@"</@movies_studio><@movies_cast>,"@DATA@"</@movies_cast><@movies_plot>,"@DATA@"</@movies_plot><@movies_notes>,"@DATA@"</@movies_notes><@vcodecs_name>,"@DATA@"</@vcodecs_name>
<!-- /ITEMS -->
