char *s = N_("<b>Example</b>");
char *s = N_("<b>Export</b>");
char *s = N_("<b>Header</b>");
char *s = N_("<b>Limits</b>");
char *s = N_("<b>Sort by</b>");
char *s = N_("<b>Split into</b>");
char *s = N_("<b>Style</b>");
char *s = N_("<b>Template</b>");
char *s = N_("Author");
char *s = N_("Black and white");
char *s = N_("Choose style file");
char *s = N_("Convert images");
char *s = N_("Data");
char *s = N_("Description");
char *s = N_("Destination");
char *s = N_("E-mail");
char *s = N_("Export");
char *s = N_("Export data");
char *s = N_("Format");
char *s = N_("Maximal height");
char *s = N_("Maximal width");
char *s = N_("Posters");
char *s = N_("Reverse");
char *s = N_("Template");
char *s = N_("Use custom file");
char *s = N_("Version");
char *s = N_("entries per page");
char *s = N_("files");
char *s = N_("loaned only");
char *s = N_("none");
char *s = N_("not loaned only");
char *s = N_("seen only");
char *s = N_("unseen only");
