char *s = N_("<b>Some Hints</b>:\n"
             "\n"
             "On the next step you can assign your csv fields with the one in Griffith.\n"
             "One CSV-field can be assigned to multiple griffith-fields, i.e. if you have only one title.\n"
             "Your cast-list should be comma-seperated to break it apart in multiple lines.");
char *s = N_("Assigned Fields (CSV > Griffith)");
char *s = N_("CSV Fields");
char *s = N_("CSV Import");
char *s = N_("Delimiter");
char *s = N_("Encoding");
char *s = N_("Fields");
char *s = N_("File-Options");
char *s = N_("From Row");
char *s = N_("Griffith Fields");
char *s = N_("If your file has no column headers then you should import from row 0.");
char *s = N_("Lineterminator");
char *s = N_("Quotechar");
char *s = N_("ascii (English)\n"
             "big5 (Traditional Chinese)\n"
             "big5hkscs (Traditional Chinese)\n"
             "cp037 (English)\n"
             "cp424 (Hebrew)\n"
             "cp437 (English)\n"
             "cp500 (Western Europe)\n"
             "cp737 (Greek)\n"
             "cp775 (Baltic languages)\n"
             "cp850 (Western Europe)\n"
             "cp852 (Central and Eastern Europe)\n"
             "cp855 (Bulgarian, Byelorussian, Macedonian, Russian, Serbian)\n"
             "cp856 (Hebrew)\n"
             "cp857 (Turkish)\n"
             "cp860 (Portuguese)\n"
             "cp861 (Icelandic)\n"
             "cp862 (Hebrew)\n"
             "cp863 (Canadian)\n"
             "cp864 (Arabic)\n"
             "cp865 (Danish, Norwegian)\n"
             "cp866 (Russian)\n"
             "cp869 (Greek)\n"
             "cp874 (Thai)\n"
             "cp875 (Greek)\n"
             "cp932 (Japanese)\n"
             "cp949 (Korean)\n"
             "cp950 (Traditional Chinese)\n"
             "cp1006 (Urdu)\n"
             "cp1026 (Turkish)\n"
             "cp1140 (Western Europe)\n"
             "cp1250 (Central and Eastern Europe)\n"
             "cp1251 (Bulgarian, Byelorussian, Macedonian, Russian, Serbian)\n"
             "cp1252 (Western Europe)\n"
             "cp1253 (Greek)\n"
             "cp1254 (Turkish)\n"
             "cp1255 (Hebrew)\n"
             "cp1256 (Arabic)\n"
             "cp1257 (Baltic languages)\n"
             "cp1258 (Vietnamese)\n"
             "euc_jp (Japanese)\n"
             "euc_jis_2004 (Japanese)\n"
             "euc_jisx0213 (Japanese)\n"
             "euc_kr (Korean)\n"
             "gb2312 (Simplified Chinese)\n"
             "gbk (Unified Chinese)\n"
             "gb18030 (Unified Chinese)\n"
             "hz (Simplified Chinese)\n"
             "iso2022_jp (Japanese)\n"
             "iso2022_jp_1 (Japanese)\n"
             "iso2022_jp_2 (Japanese, Korean, Simplified Chinese, Western Europe, Greek)\n"
             "iso2022_jp_2004 (Japanese)\n"
             "iso2022_jp_3 (Japanese)\n"
             "iso2022_jp_ext (Japanese)\n"
             "iso2022_kr (Korean)\n"
             "latin_1 (West Europe)\n"
             "iso8859_2 (Central and Eastern Europe)\n"
             "iso8859_3 (Esperanto, Maltese)\n"
             "iso8859_4 (Baltic languagues)\n"
             "iso8859_5 (Bulgarian, Byelorussian, Macedonian, Russian, Serbian)\n"
             "iso8859_6 (Arabic)\n"
             "iso8859_7 (Greek)\n"
             "iso8859_8 (Hebrew)\n"
             "iso8859_9 (Turkish)\n"
             "iso8859_10 (Nordic languages)\n"
             "iso8859_13 (Baltic languages)\n"
             "iso8859_14 (Celtic languages)\n"
             "iso8859_15 (Western Europe)\n"
             "johab (Korean)\n"
             "koi8_r (Russian)\n"
             "koi8_u (Ukrainian)\n"
             "mac_cyrillic (Bulgarian, Byelorussian, Macedonian, Russian, Serbian)\n"
             "mac_greek (Greek)\n"
             "mac_iceland (Icelandic)\n"
             "mac_latin2 (Central and Eastern Europe)\n"
             "mac_roman (Western Europe)\n"
             "mac_turkish (Turkish)\n"
             "ptcp154 (Kazakh)\n"
             "shift_jis (Japanese)\n"
             "shift_jis_2004 (Japanese)\n"
             "shift_jisx0213 (Japanese)\n"
             "utf_16 (all languages)\n"
             "utf_16_be (all languages (BMP only))\n"
             "utf_16_le (all languages (BMP only))\n"
             "utf_7 (all languages)\n"
             "utf_8 (all languages)");
