<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<title><@header>@DATA@</@header></title>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<meta name="generator" content="Griffith HTML export plugin" />
	<meta name="author" content="Piotr Ozarowski" />
<@style>	<link rel="stylesheet" type="text/css" href="./@DATA@" /></@style>
</head>
<body>
<@header><h1>@DATA@</h1></@header>

<!-- ITEMS -->
<table<@id> id="i@DATA@"</@id>><@movies_image>
	<tr class="image">
		<td<@rowspan> rowspan="@DATA@"</@rowspan>>
			<img src="./posters/@DATA@" alt="@TITLE@" />
		</td>
	</tr></@movies_image><@movies_title>
	<tr class="title">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_title><@movies_o_title>
	<tr class="otitle">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_o_title><@movies_number>
	<tr class="number">		
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_number><@movies_year>
	<tr class="year">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_year><@movies_director>
	<tr class="director">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_director><@movies_rating>
	<tr class="rating">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_rating><@movies_runtime>
	<tr class="runtime">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_runtime><@movies_country>
	<tr class="country">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_country><@movies_genre>
	<tr class="genre">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_genre><@movies_site>
	<tr class="links">
		<th>@TITLE@</th>
		<td><a href="@DATA@">@DATA@</a></td>
	</tr></@movies_site><@movies_o_site>
	<tr class="links">
		<th>@TITLE@</th>
		<td><a href="@DATA@">@DATA@</a></td>
	</tr></@movies_o_site><@movies_trailer>
	<tr class="links">
		<th>@TITLE@</th>
		<td><a href="@DATA@">@DATA@</a></td>
	</tr></@movies_trailer><@media_name>
	<tr class="media">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@media_name><@movies_media_num>
	<tr class="media">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_media_num><@vcodecs_name>
	<tr class="media">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@vcodecs_name><@collections_name>
	<tr class="collection">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@collections_name><@volumes_name>
	<tr class="volume">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@volumes_name><@movies_seen>
	<tr class="seen">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_seen><@movies_loaned>
	<tr class="loaned">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_loaned><@movies_classification>
	<tr class="classification">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_classification><@movies_studio>
	<tr class="studio">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_studio><@movies_cast>
	<tr class="cast">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_cast><@movies_plot>
	<tr class="plot">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_plot><@movies_notes>
	<tr class="notes">
		<th>@TITLE@</th>
		<td>@DATA@</td>
	</tr></@movies_notes>
</table>
<!-- /ITEMS -->

<@navigation>@DATA@</@navigation>
<@copyright><div class="copyright">@DATA@</div></@copyright>
</body>
</html>
