<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<title><@header>@DATA@</@header></title>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<meta name="generator" content="Griffith HTML export plugin" />
	<meta name="author" content="Piotr Ozarowski" /><@style>
	<link rel="stylesheet" type="text/css" href="./@DATA@" /></@style>
</head>
<body>
<@header><h1>@DATA@</h1></@header>

<table>
	<thead>
		<tr><@movies_image>
			<th>@TITLE@</th></@movies_image><@movies_number>
			<th>@TITLE@</th></@movies_number><@movies_title>
			<th>@TITLE@</th></@movies_title><@movies_o_title>
			<th>@TITLE@</th></@movies_o_title><@movies_year>
			<th>@TITLE@</th></@movies_year><@movies_director>
			<th>@TITLE@</th></@movies_director><@movies_rating>
			<th>@TITLE@</th></@movies_rating><@movies_runtime>
			<th>@TITLE@</th></@movies_runtime><@movies_country>
			<th>@TITLE@</th></@movies_country><@movies_genre>
			<th>@TITLE@</th></@movies_genre><@movies_site>
			<th>@TITLE@</th></@movies_site><@movies_o_site>
			<th>@TITLE@</th></@movies_o_site><@movies_trailer>
			<th>@TITLE@</th></@movies_trailer><@media_name>
			<th>@TITLE@</th></@media_name><@movies_media_num>
			<th>@TITLE@</th></@movies_media_num><@vcodecs_name>
			<th>@TITLE@</th></@vcodecs_name><@collections_name>
			<th>@TITLE@</th></@collections_name><@volumes_name>
			<th>@TITLE@</th></@volumes_name><@movies_seen>
			<th>@TITLE@</th></@movies_seen><@movies_loaned>
			<th>@TITLE@</th></@movies_loaned><@movies_classification>
			<th>@TITLE@</th></@movies_classification><@movies_studio>
			<th>@TITLE@</th></@movies_studio><@movies_cast>
			<th>@TITLE@</th></@movies_cast><@movies_plot>
			<th>@TITLE@</th></@movies_plot><@movies_notes>
			<th>@TITLE@</th></@movies_notes>
		</tr>
	</thead>

<!-- ITEMS -->
	<tr<@id> id="i@DATA@"</@id>><@movies_image>
		<td class="image"><img src="./posters/@DATA@" alt="poster" /></td></@movies_image><@movies_number>
		<td class="number">@DATA@</td></@movies_number><@movies_title>
		<td class="title">@DATA@</td></@movies_title><@movies_o_title>
		<td class="otitle">@DATA@</td></@movies_o_title><@movies_year>
		<td class="year">@DATA@</td></@movies_year><@movies_director>
		<td class="director">@DATA@</td></@movies_director><@movies_rating>
		<td class="rating">@DATA@</td></@movies_rating><@movies_runtime>
		<td class="runtime">@DATA@</td></@movies_runtime><@movies_country>
		<td class="country">@DATA@</td></@movies_country><@movies_genre>
		<td class="genre">@DATA@</td></@movies_genre><@movies_site>
		<td class="links"><a href="@DATA@">link</a></td></@movies_site><@movies_o_site>
		<td class="links"><a href="@DATA@">link</a></td></@movies_o_site><@movies_trailer>
		<td class="links"><a href="@DATA@">link</a></td></@movies_trailer><@media_name>
		<td class="media">@DATA@</td></@media_name><@movies_media_num>
		<td class="media">@DATA@</td></@movies_media_num><@vcodecs_name>
		<td class="media">@DATA@</td></@vcodecs_name><@collections_name>
		<td class="media">@DATA@</td></@collections_name><@volumes_name>
		<td class="media">@DATA@</td></@volumes_name><@movies_seen>
		<td class="seen">@DATA@</td></@movies_seen><@movies_loaned>
		<td class="loaned">@DATA@</td></@movies_loaned><@movies_classification>
		<td class="classification">@DATA@</td></@movies_classification><@movies_studio>
		<td class="studio">@DATA@</td></@movies_studio><@movies_cast>
		<td class="cast">@DATA@</td></@movies_cast><@movies_plot>
		<td class="plot">@DATA@</td></@movies_plot><@movies_notes>
		<td class="notes">@DATA@</td></@movies_notes>
	</tr>
<!-- /ITEMS -->

</table>
<@navigation>@DATA@</@navigation>
<@copyright><div class="copyright">@DATA@</div></@copyright>
</body>
</html>
