char *s = N_("0 means no limits");
char *s = N_("<b>Audio channels</b>");
char *s = N_("<b>Audio codecs</b>");
char *s = N_("<b>Audio</b>");
char *s = N_("<b>Collection</b>");
char *s = N_("<b>Defaults</b>");
char *s = N_("<b>E-mail reminder</b>");
char *s = N_("<b>Languages</b>");
char *s = N_("<b>Loan History</b>");
char *s = N_("<b>Media</b>");
char *s = N_("<b>Search for:</b>");
char *s = N_("<b>Search results</b>");
char *s = N_("<b>Select database to use</b>");
char *s = N_("<b>Show only the following columns</b>");
char *s = N_("<b>Sort by</b>");
char *s = N_("<b>Subtitle formats</b>");
char *s = N_("<b>Subtitles</b>");
char *s = N_("<b>Tags</b>");
char *s = N_("<b>Use spell checking on the following fields:</b>");
char *s = N_("<b>Video codecs</b>");
char *s = N_("<b>Volume</b>");
char *s = N_("Add");
char *s = N_("Add a new movie");
char *s = N_("Add a new person");
char *s = N_("Add movie");
char *s = N_("Add movie and close");
char *s = N_("Add new movie");
char *s = N_("All Movies");
char *s = N_("Attempt to fetch a big poster from Amazon services Web site");
char *s = N_("By Collection");
char *s = N_("By Tag");
char *s = N_("By Volume");
char *s = N_("Cast");
char *s = N_("Choose Image");
char *s = N_("Classification");
char *s = N_("Clears filter");
char *s = N_("Clone");
char *s = N_("Collection");
char *s = N_("Color");
char *s = N_("Condition");
char *s = N_("Country");
char *s = N_("DVD layers");
char *s = N_("DVD region");
char *s = N_("Database");
char *s = N_("Delete a person");
char *s = N_("Details");
char *s = N_("Dictionaries");
char *s = N_("Director");
char *s = N_("Discs");
char *s = N_("E-mail");
char *s = N_("E-mail reminder");
char *s = N_("E_xport");
char *s = N_("Edit a person");
char *s = N_("Edit movie details");
char *s = N_("Favorite Movie Plugin");
char *s = N_("Fetch from Amazon");
char *s = N_("Fill empty fields only");
char *s = N_("Filter ");
char *s = N_("Font for reports");
char *s = N_("Forum");
char *s = N_("Found Results");
char *s = N_("Genre");
char *s = N_("Get from web");
char *s = N_("Get movie data from:");
char *s = N_("Griffith");
char *s = N_("Homepage");
char *s = N_("Host");
char *s = N_("I_mport");
char *s = N_("Image");
char *s = N_("Include Front Image");
char *s = N_("Include movie number");
char *s = N_("Languages");
char *s = N_("Layers");
char *s = N_("Limit search results to");
char *s = N_("Load poster from disk");
char *s = N_("Loan");
char *s = N_("Loan Management");
char *s = N_("Loan movie");
char *s = N_("Loan to");
char *s = N_("Loaned Movies");
char *s = N_("Loaned to");
char *s = N_("Loans");
char *s = N_("Main list");
char *s = N_("Manage People");
char *s = N_("Media");
char *s = N_("Medium");
char *s = N_("Meter\n"
             "Stars");
char *s = N_("Movie details");
char *s = N_("Movie information");
char *s = N_("Name");
char *s = N_("Needs authentication");
char *s = N_("Not Seen Movies");
char *s = N_("Notes");
char *s = N_("Number");
char *s = N_("Official site");
char *s = N_("Original Title");
char *s = N_("Original title");
char *s = N_("Other");
char *s = N_("PDF reader");
char *s = N_("Password");
char *s = N_("People");
char *s = N_("Phone");
char *s = N_("Please wait a moment...");
char *s = N_("Plot");
char *s = N_("Plugins");
char *s = N_("Port");
char *s = N_("Poster image");
char *s = N_("Poster viewer");
char *s = N_("Preferences");
char *s = N_("Print Cover");
char *s = N_("Print cover");
char *s = N_("Rating");
char *s = N_("Rating image");
char *s = N_("Region");
char *s = N_("Remove");
char *s = N_("Remove selected movie");
char *s = N_("Removes poster");
char *s = N_("Rename");
char *s = N_("Report bug");
char *s = N_("Return");
char *s = N_("Reverse");
char *s = N_("Runtime");
char *s = N_("SMTP port");
char *s = N_("SMTP server");
char *s = N_("Searching on the web");
char *s = N_("Seen it");
char *s = N_("Select A File");
char *s = N_("Select first movie");
char *s = N_("Select last movie");
char *s = N_("Select next movie");
char *s = N_("Select previous movie");
char *s = N_("Sender E-mail address");
char *s = N_("Show other movies in this collection");
char *s = N_("Show other movies in this volume");
char *s = N_("Simple");
char *s = N_("Site");
char *s = N_("Size");
char *s = N_("Spell check the following language");
char *s = N_("Standard\n"
             "Slim\n"
             "Double Slim");
char *s = N_("Studio");
char *s = N_("Suggest an unseen movie");
char *s = N_("Tags");
char *s = N_("Technical information");
char *s = N_("This movie is part of...");
char *s = N_("Title");
char *s = N_("Toolbar");
char *s = N_("Trailer");
char *s = N_("Type");
char *s = N_("Type in the original film tile, select source and choose <i>Get from web</i>. Griffith will try to fetch all the related information from the Web.");
char *s = N_("Use Spellchecker");
char *s = N_("Use TLS?");
char *s = N_("Use locale:");
char *s = N_("Username");
char *s = N_("Video codec");
char *s = N_("Visit movie web site");
char *s = N_("Visit the oficial web site");
char *s = N_("Volume");
char *s = N_("Watch the trailer");
char *s = N_("Year");
char *s = N_("_Edit");
char *s = N_("_File");
char *s = N_("_Help");
char *s = N_("_Loans");
char *s = N_("_Tools");
char *s = N_("_View");
char *s = N_("add entered name to collections list");
char *s = N_("add entered name to volumes list");
char *s = N_("classification");
char *s = N_("click here if movie was returned");
char *s = N_("collection details");
char *s = N_("loans movie or volume");
char *s = N_("minutes");
char *s = N_("movies");
char *s = N_("remove selected collection");
char *s = N_("remove selected volume");
char *s = N_("tags");
char *s = N_("volume details");
