char *s = N_("<b>Import only fields selected below</b>");
char *s = N_("<b>Importing...</b>");
char *s = N_("Author");
char *s = N_("Description");
char *s = N_("E-mail");
char *s = N_("Fields");
char *s = N_("Import window");
char *s = N_("Plugin");
char *s = N_("Source");
char *s = N_("Version");
