<?xml version="1.0" encoding="UTF-8" standalone='yes' ?>
<movies>
<!-- ITEMS -->
	<movie><@id>
		<id>@DATA@</id></@id><@movies_image>
		<image>@DATA@</image></@movies_image><@movies_title>
		<title>@DATA@</title></@movies_title><@movies_o_title>
		<otitle>@DATA@</otitle></@movies_o_title><@movies_number>
		<number>@DATA@</number></@movies_number><@movies_year>
		<year>@DATA@</year></@movies_year><@movies_director>
		<director>@DATA@</director></@movies_director><@movies_rating>
		<rating>@DATA@</rating></@movies_rating><@movies_runtime>
		<runtime>@DATA@</runtime></@movies_runtime><@movies_country>
		<country>@DATA@</country></@movies_country><@movies_genre>
		<genre>@DATA@</genre></@movies_genre><@movies_site>
		<imdb>@DATA@</imdb></@movies_site><@movies_o_site>
		<site>@DATA@</site></@movies_o_site><@movies_trailer>
		<trailer>@DATA@</trailer></@movies_trailer><@media_name>
		<media_name>@DATA@</media_name></@media_name><@movies_media_num>
		<media>@DATA@</media></@movies_media_num><@vcodecs_name>
		<vcodec_name>@DATA@</vcodec_name></@vcodecs_name><@collections_name>
		<collection>@DATA@</collection></@collections_name><@volumes_name>
		<volume>@DATA@</volume></@volumes_name><@movies_seen>
		<seen>@DATA@</seen></@movies_seen><@movies_loaned>
		<loaned>@DATA@</loaned></@movies_loaned><@movies_classification>
		<classification>@DATA@</classification></@movies_classification><@movies_studio>
		<studio>@DATA@</studio></@movies_studio><@movies_cast>
		<cast>@DATA@</cast></@movies_cast><@movies_plot>
		<plot>@DATA@</plot></@movies_plot><@movies_notes>
		<notes>@DATA@</notes></@movies_notes>
	</movie>
<!-- /ITEMS -->
</movies>

